@@ -6,7 +6,9 @@
 # See https://github.com/nexB/scancode-toolkit for support or download.
 # See https://aboutcode.org for more information about nexB OSS projects.
 #
+import attr
 import csv
+import warnings
 
 import saneyaml
 
@@ -37,6 +39,12 @@ if TRACE:
         return logger.debug(' '.join(isinstance(a, str)
                                      and a or repr(a) for a in args))
 
+DEPRECATED_MSG = (
+    'The --csv option is deprecated and will be replaced by new CSV and '
+    'tabular output formats in the next ScanCode release. '
+    'Visit https://github.com/nexB/scancode-toolkit/issues/3043 to provide inputs and feedback.'
+)
+
 
 @output_impl
 class CsvOutput(OutputPlugin):
@@ -45,7 +53,7 @@ class CsvOutput(OutputPlugin):
         PluggableCommandLineOption(('--csv',),
             type=FileOptionType(mode='w', encoding='utf-8', lazy=True),
             metavar='FILE',
-            help='Write scan output as CSV to FILE.',
+            help='[DEPRECATED] Write scan output as CSV to FILE. ' + DEPRECATED_MSG,
             help_group=OUTPUT_GROUP,
             sort_order=30),
     ]
@@ -54,6 +62,14 @@ class CsvOutput(OutputPlugin):
         return csv
 
     def process_codebase(self, codebase, csv, **kwargs):
+        warnings.warn(
+            DEPRECATED_MSG,
+            DeprecationWarning,
+            stacklevel=1
+        )
+        import click
+        click.secho('[DEPRECATION WARNING] ' + DEPRECATED_MSG, err=True)
+
         results = self.get_files(codebase, **kwargs)
         write_csv(results, csv)
 
@@ -113,7 +129,7 @@ def flatten_scan(scan, headers):
 
         errors = scanned_file.pop('scan_errors', [])
 
-        file_info = dict(Resource=path)
+        file_info = dict(path=path)
         file_info.update(((k, v) for k, v in scanned_file.items()
         # FIXME: info are NOT lists: lists are the actual scans
                           if not isinstance(v, (list, dict))))
@@ -124,12 +140,12 @@ def flatten_scan(scan, headers):
         yield file_info
 
         for lic_exp in scanned_file.get('license_expressions', []):
-            inf = dict(Resource=path, license_expression=lic_exp)
+            inf = dict(path=path, license_expression=lic_exp)
             collect_keys(inf, 'license_expression')
             yield inf
 
         for licensing in scanned_file.get('licenses', []):
-            lic = dict(Resource=path)
+            lic = dict(path=path)
             for k, val in licensing.items():
                 # do not include matched text for now.
                 if k == 'matched_text':
@@ -139,7 +155,7 @@ def flatten_scan(scan, headers):
                     for mrk, mrv in val.items():
                         if mrk in ('match_coverage', 'rule_relevance'):
                             # normalize the string representation of this number
-                            mrv = '{:.2f}'.format(mrv)
+                            mrv = with_two_decimals(mrv)
                         else:
                             mrv = pretty(mrv)
                         mrk = 'matched_rule__' + mrk
@@ -147,8 +163,7 @@ def flatten_scan(scan, headers):
                     continue
 
                 if k == 'score':
-                    # normalize score with two decimal values
-                    val = '{:.2f}'.format(val)
+                    val = with_two_decimals(val)
 
                 # lines are present in multiple scans: keep their column name as
                 # not scan-specific. Prefix othe columns with license__
@@ -159,47 +174,58 @@ def flatten_scan(scan, headers):
             yield lic
 
         for copyr in scanned_file.get('copyrights', []):
-            inf = dict(Resource=path)
-            inf['copyright'] = copyr['value']
+            inf = dict(path=path)
+            inf['copyright'] = copyr['copyright']
             inf['start_line'] = copyr['start_line']
             inf['end_line'] = copyr['start_line']
             collect_keys(inf, 'copyright')
             yield inf
 
         for copyr in scanned_file.get('holders', []):
-            inf = dict(Resource=path)
-            inf['copyright_holder'] = copyr['value']
+            inf = dict(path=path)
+            inf['holder'] = copyr['holder']
             inf['start_line'] = copyr['start_line']
             inf['end_line'] = copyr['start_line']
             collect_keys(inf, 'copyright')
             yield inf
 
         for copyr in scanned_file.get('authors', []):
-            inf = dict(Resource=path)
-            inf['author'] = copyr['value']
+            inf = dict(path=path)
+            inf['author'] = copyr['author']
             inf['start_line'] = copyr['start_line']
             inf['end_line'] = copyr['start_line']
             collect_keys(inf, 'copyright')
             yield inf
 
         for email in scanned_file.get('emails', []):
-            email_info = dict(Resource=path)
+            email_info = dict(path=path)
             email_info.update(email)
             collect_keys(email_info, 'email')
             yield email_info
 
         for url in scanned_file.get('urls', []):
-            url_info = dict(Resource=path)
+            url_info = dict(path=path)
             url_info.update(url)
             collect_keys(url_info, 'url')
             yield url_info
 
-        for package in scanned_file.get('packages', []):
+        for package in scanned_file.get('package_data', []):
             flat = flatten_package(package, path)
             collect_keys(flat, 'package')
             yield flat
 
 
+def with_two_decimals(val):
+    """
+    Return a normalized score string with two decimal values
+    """
+    if isinstance(val, (float, int)):
+        val = '{:.2f}'.format(val)
+    if not isinstance(val, str):
+        val = str(val)
+    return val
+
+
 def pretty(data):
     """
     Return a unicode text pretty representation of data (as YAML or else) if
@@ -229,7 +255,9 @@ def get_package_columns(_columns=set()):
     if _columns:
         return _columns
 
-    from packagedcode.models import Package
+    from packagedcode.models import PackageData
+
+    package_data_fields = [field.name for field in attr.fields(PackageData)]
 
     # exclude some columns for now that contain list of items
     excluded_columns = {
@@ -252,7 +280,7 @@ def get_package_columns(_columns=set()):
         'notice_url',
     ]
 
-    fields = Package.fields() + extra_columns
+    fields = package_data_fields + extra_columns
     _columns = set(f for f in fields if f not in excluded_columns)
     return _columns
 
@@ -263,7 +291,7 @@ def flatten_package(_package, path, prefix='package__'):
 
     package_columns = get_package_columns()
 
-    pack = dict(Resource=path)
+    pack = dict(path=path)
     for k, val in _package.items():
         if k not in package_columns:
             continue
